/*
 * rhythmarty-plugin.c
 * 
 * Copyright (C) 2010 Sinew Software Systems
 * Authers: Vinod Kumar Dahiya
 *          Hemant Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The Rhythmbox authors hereby grant permission for non-GPL compatible
 * GStreamer plugins to be used and distributed together with GStreamer
 * and Rhythmbox. This permission is above and beyond the permissions granted
 * by the GPL license by which Rhythmbox is covered. If you modify this code
 * you may extend this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this exception
 * statement from your version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>
#include <gmodule.h>
#include <gconf/gconf-client.h>
#include "rhythmarty-plugin.h"
#include "rhythmarty.h"
#include "coverart.h"

#define COVER_CACHE_DIRNAME "rhythmbox/rhythmarty/albums"
//############################# Misc fuctions ##################################

#define RHYTHMARTY_STOCK_ICON "rhythmarty-stock-icon"

typedef struct
{
  char *stock_id;
  char *icon_name;
} StockIcon;

void
rb_arty_stock_icons_init (void)
{
  GtkIconFactory *factory;
  int i;
  static gboolean done = FALSE;

  StockIcon items[] =
  {
    { RHYTHMARTY_STOCK_ICON,   "rhythmarty.svg"   }
  };

  if (done)
    return;

  done = TRUE;
  
  factory = gtk_icon_factory_new ();
  gtk_icon_factory_add_default (factory);

  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
    {
      GtkIconSet *icon_set;
      GdkPixbuf *pixbuf;

	  gchar *filename;
	  filename = g_strconcat(PLUGINDIR,"/rhythmarty/",items[i].icon_name,NULL);

      pixbuf = gdk_pixbuf_new_from_file (filename,NULL);

      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
      gtk_icon_set_unref (icon_set);
		
      g_object_unref (G_OBJECT (pixbuf));
    }

  g_object_unref (G_OBJECT (factory));
}

//################ Update visible state of various widgets #####################
static void 
rb_arty_update_additional_widgets(RBArtyPlugin *plugin,
	       RBShell *shell)
{
	if(plugin->priv->showing_browser&&plugin->priv->showing_genre)
	{
		gtk_widget_show (plugin->priv->genre_view);
	}
	else
	{
		gtk_widget_hide (plugin->priv->genre_view);
	}

	if(plugin->priv->showing_browser)
	{
		gtk_widget_show (plugin->priv->vpane);
	}
	else
	{
		gtk_widget_hide (plugin->priv->vpane);
	}

	if(plugin->priv->showing_plugin)
	{
		switch(plugin->priv->state)
		{
			case RHYTHMARTY_SHOWING:
				//gtk_widget_show_all (plugin->priv->show_browser);
				gtk_action_set_visible (GTK_ACTION(plugin->priv->showhidebrowser_action),TRUE);
				gtk_widget_show_all (plugin->priv->show_songs);
				gtk_widget_hide_all (plugin->priv->show_albums);
				break;

			case RHYTHMARTY_SONGLIST:
				//gtk_widget_hide_all (plugin->priv->show_browser);
				gtk_action_set_visible (GTK_ACTION(plugin->priv->showhidebrowser_action),FALSE);
				gtk_widget_hide_all (plugin->priv->show_songs);
				gtk_widget_show_all (plugin->priv->show_albums);
				break;

			case RHYTHMARTY_OTHER:
				gtk_action_set_visible (GTK_ACTION(plugin->priv->showhidebrowser_action),FALSE);
				gtk_widget_hide_all (plugin->priv->show_songs);
				gtk_widget_hide_all (plugin->priv->show_albums);
				break;

			default:
				break;
		}
	}
	else
	{
		gtk_action_set_visible (GTK_ACTION(plugin->priv->showhidebrowser_action),FALSE);
		gtk_widget_hide_all (plugin->priv->show_songs);
		gtk_widget_hide_all (plugin->priv->show_albums);
	}
}

static void
rb_arty_update_visible_state(RBArtyPlugin *plugin,
	       RBShell *shell)
{
	RBSource *source;
	g_object_get(shell,"selected-source",&source,NULL);

	if(plugin->priv->showing_plugin && source == plugin->priv->library_source
	   && !plugin->priv->showing_songs)
	{
		//activate page
		rb_shell_notebook_set_page (shell,plugin->priv->main_widget);
		plugin->priv->showing_plugin = TRUE;
		plugin->priv->state = RHYTHMARTY_SHOWING;

		//make search header visible on the page
		RBSourceHeader *header;
		g_object_get (shell, "source-header" , &header, NULL);
		rb_source_header_set_source(header,plugin->priv->library_source);
	}

	if(source != plugin->priv->library_source)
		plugin->priv->state = RHYTHMARTY_OTHER;
}

//################# Button and state state changed callbacks ###################

static void
rb_arty_show_albums_clicked (GtkButton *button, RBArtyPlugin *plugin)
{
	plugin->priv->showing_songs = FALSE;
	rb_arty_update_visible_state(plugin,plugin->priv->shell);
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

void
rb_arty_show_songs_clicked (GtkButton *button, RBArtyPlugin *plugin)
{
	plugin->priv->showing_songs = TRUE;
	rb_shell_notebook_set_page (plugin->priv->shell,
	                            GTK_WIDGET(plugin->priv->library_source));
	plugin->priv->state = RHYTHMARTY_SONGLIST;
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);

	//set browser gutter -- needed because sometimes gutter size not updated --
	GList *views = rb_source_get_property_views (plugin->priv->library_source);
	GtkWidget *pane = gtk_widget_get_parent 
		(gtk_widget_get_parent(GTK_WIDGET(views->data)));
	GTK_PANED(pane)->child1_shrink=1;
	gtk_paned_set_position(GTK_PANED(pane),0);
}

static void
rb_arty_source_changed(RBSourceList *list,GObject *source,RBArtyPlugin *plugin)
{
	rb_arty_update_visible_state(plugin,plugin->priv->shell);
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

static void
rb_arty_notebook_page_changed(GtkNotebook     *notebook,
                              GtkNotebookPage *page,
                              guint            page_num,
                              RBArtyPlugin *plugin)
{
	GtkWidget *current_page = 
		gtk_notebook_get_nth_page (notebook,
		                          page_num); 

	if(current_page == GTK_WIDGET(plugin->priv->library_source))
	{
		plugin->priv->state = RHYTHMARTY_SONGLIST;
	}
	
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

//############################### Show/Hide Menu/Tool item #####################

static void
rb_arty_force_library_browser(RBArtyPlugin *plugin, RBShell *shell)
{
	//force browser on & hide its option
	GtkUIManager *ui_manager;
	ui_manager = (GtkUIManager *)rb_shell_get_ui_manager (shell);

	GtkAction *action = gtk_ui_manager_get_action (ui_manager,"/ui/ToolBar/Browse");
	plugin->priv->library_browser_visible = 
		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action));
	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(action),TRUE);
	gtk_action_set_visible (GTK_ACTION(action),FALSE);
	
	//get and set browser gutter 
	GList *views = rb_source_get_property_views (plugin->priv->library_source);
	GtkWidget *pane = gtk_widget_get_parent 
		(gtk_widget_get_parent(GTK_WIDGET(views->data)));
	GTK_PANED(pane)->child1_shrink=1;
	plugin->priv->library_browser_pos = gtk_paned_get_position(GTK_PANED(pane));
	gtk_paned_set_position(GTK_PANED(pane),0);
}

static void
rb_arty_restore_library_browser(RBArtyPlugin *plugin, RBShell *shell)
{
	//restore browser option
	GtkUIManager *ui_manager;
	ui_manager = (GtkUIManager *)rb_shell_get_ui_manager (shell);

	GtkAction *action = gtk_ui_manager_get_action (ui_manager,"/ui/ToolBar/Browse"); 
	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(action),plugin->priv->library_browser_visible);
	gtk_action_set_visible (GTK_ACTION(action),TRUE);
	
	//get and set browser gutter 
	GList *views = rb_source_get_property_views (plugin->priv->library_source);
	GtkWidget *pane = gtk_widget_get_parent 
		(gtk_widget_get_parent(GTK_WIDGET(views->data)));
	GTK_PANED(pane)->child1_shrink=0;
	gtk_paned_set_position(GTK_PANED(pane),plugin->priv->library_browser_pos);
}


static void
rb_arty_browser_toggled (GtkToggleAction *toggleaction, RBArtyPlugin *plugin)
{
	plugin->priv->showing_browser = gtk_toggle_action_get_active (toggleaction);
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

static void
rb_arty_showhide_toggled(GtkToggleAction *toggleaction, RBArtyPlugin *plugin)
{
	if(plugin->priv->showing_plugin)
	{
		if(plugin->priv->showing_browser)
		{
			plugin->priv->browser_size = 
				gtk_paned_get_position (GTK_PANED(plugin->priv->hpane));
		}
		
		//disconnect notebook page switch handler
		g_signal_handler_disconnect 
			(G_OBJECT(gtk_widget_get_parent (plugin->priv->main_widget)),
			          plugin->priv->page_switched_id);

		//remove main widget, this try to destroy the widget but we already 
		//ref once at activation to prevent it
		rb_shell_remove_widget(plugin->priv->shell,plugin->priv->main_widget,RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK);
		plugin->priv->showing_plugin = FALSE;
		plugin->priv->state = RHYTHMARTY_OTHER;

		//if current source is library then set it as current page, otherwise
		//some ugly blank library page will come
		RBSource *source;
		g_object_get(plugin->priv->shell,"selected-source",&source,NULL);

		if(source == plugin->priv->library_source)
		{
			rb_shell_notebook_set_page (plugin->priv->shell,
			                            GTK_WIDGET(plugin->priv->library_source));
			plugin->priv->state = RHYTHMARTY_SONGLIST;
		}

		//restore library browser to previous state
		rb_arty_restore_library_browser(plugin,plugin->priv->shell);
	}
	else
	{
		//add to shell main notebook 
		rb_shell_add_widget(plugin->priv->shell,plugin->priv->main_widget,RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK,TRUE,TRUE);
		gtk_widget_show_all (plugin->priv->main_widget);
		plugin->priv->showing_plugin = TRUE;

		//show if required
		rb_arty_update_visible_state(plugin,plugin->priv->shell);

		//get notebook page switch notification
		plugin->priv->page_switched_id = 
			g_signal_connect(G_OBJECT(gtk_widget_get_parent (plugin->priv->main_widget)),
			                          "switch-page",
			                          G_CALLBACK(rb_arty_notebook_page_changed),
			                          plugin);

		//force library browser to show. this is required hack to 
		//oprate out plugin properly
		rb_arty_force_library_browser(plugin,plugin->priv->shell);
		
		//set browser size doesn't works
		//if(plugin->priv->browser_size)
		//	gtk_paned_set_position (GTK_PANED(plugin->priv->hpane),
		//	                        plugin->priv->browser_size);
	}

	//update visible state of buttons and views
	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

/*const char *showhide_xml_ui = 
	"<ui>" 
  "<menubar name=\"MenuBar\"> "
    "<menu name=\"ViewMenu\" action=\"View\"> "
       "  <menuitem name=\"ViewRhythmArty\" action=\"ViewRhythmArtyAction\"/>"
    "</menu>"
  "</menubar>"
"<toolbar name=\"ToolBar\">"
    "<toolitem name=\"ViewRhythmArty\" action=\"ViewRhythmArtyAction\"/>"
"</toolbar>"
"</ui>";*/

const char *showhide_xml_ui = 
	"<ui>" 
  "<menubar name=\"MenuBar\"> "
    "<menu name=\"ViewMenu\" action=\"View\"> "
		"<separator/>"
        "  <menuitem name=\"ViewRhythmArty\" action=\"ViewRhythmArtyAction\"/>"
		"  <menuitem name=\"ViewRhythmArtyBrowser\" action=\"ViewRhythmArtyBrowserAction\"/>"
    "</menu>"
	"<menu name=\"ToolsMenu\" action=\"Tools\"> "
		"  <menuitem name=\"GetRhythmArtyCovers\" action=\"GetRhythmArtyCoversAction\"/>"
    "</menu>"
  "</menubar>"
"<toolbar name=\"ToolBar\">"
    "<toolitem name=\"ViewRhythmArty\" action=\"ViewRhythmArtyAction\"/>"
"</toolbar>"
"</ui>";


static void
rb_arty_craete_showhide_items(RBArtyPlugin *plugin,
	       RBShell *shell)
{
	GtkAction *menu_action = 
		gtk_action_new ("RhythmArtyMenuAction",_("RhythmArty"),
	    _("RhythmArty"),NULL);
	
	plugin->priv->showhide_action = 
		gtk_toggle_action_new ("ViewRhythmArtyAction",_("RhythmArty Browser"),
	    _("RhythmArty Browser"),RHYTHMARTY_STOCK_ICON);

	g_signal_connect(G_OBJECT(plugin->priv->showhide_action),"toggled",
	                 G_CALLBACK(rb_arty_showhide_toggled),
	                 plugin);

	plugin->priv->showhidebrowser_action = 
		gtk_toggle_action_new ("ViewRhythmArtyBrowserAction",_("Artist/Genre Browser"),
	    _("Artist/Genre Browser"),GTK_STOCK_INDEX);

	g_signal_connect(G_OBJECT(plugin->priv->showhidebrowser_action),"toggled",
	                 G_CALLBACK(rb_arty_browser_toggled),
	                 plugin);

	plugin->priv->get_covers_action = 
		gtk_action_new ("GetRhythmArtyCoversAction",_("Get Covers"),
	    _("Get Covers"),NULL);

	g_signal_connect(G_OBJECT(plugin->priv->get_covers_action),"activate",
	                 G_CALLBACK(rb_arty_get_covers_clicked),
	                 plugin);

	plugin->priv->showhide_actiongroup = 
		gtk_action_group_new ("ViewRhythmArtyActionGroup");

	gtk_action_group_add_action 
		(plugin->priv->showhide_actiongroup, GTK_ACTION(menu_action));
	gtk_action_group_add_action 
		(plugin->priv->showhide_actiongroup, GTK_ACTION(plugin->priv->showhide_action));
	gtk_action_group_add_action 
		(plugin->priv->showhide_actiongroup, GTK_ACTION(plugin->priv->showhidebrowser_action));
	gtk_action_group_add_action 
		(plugin->priv->showhide_actiongroup, GTK_ACTION(plugin->priv->get_covers_action));
	
	GtkUIManager *ui_manager;
	ui_manager = (GtkUIManager *)rb_shell_get_ui_manager (shell);
	gtk_ui_manager_insert_action_group (ui_manager,plugin->priv->showhide_actiongroup,0);

	plugin->priv->showhide_ui_merge_id =
		gtk_ui_manager_add_ui_from_string (ui_manager,showhide_xml_ui,
	                                   strlen(showhide_xml_ui),NULL);
}

static void
rb_arty_remove_showhide_items(RBArtyPlugin *plugin,
	       RBShell *shell)
{
	GtkUIManager *ui_manager;
	ui_manager = (GtkUIManager *)rb_shell_get_ui_manager (shell);

	gtk_ui_manager_remove_ui (ui_manager,plugin->priv->showhide_ui_merge_id);
	gtk_ui_manager_remove_action_group (ui_manager,plugin->priv->showhide_actiongroup);
}


//########################## GConf stuff #######################################
static void 
rb_arty_genre_key_changed (GConfClient * client,
                              guint cnxn_id,
                              GConfEntry * entry,
                              RBArtyPlugin * plugin)
{
	gint value;

	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
		return;

	value = gconf_value_get_int (entry->value);

	plugin->priv->showing_genre = (value > 0);

	rb_arty_update_additional_widgets(plugin,plugin->priv->shell);
}

static void 
rb_arty_load_configuartion (GConfClient * client,RBArtyPlugin * plugin)
{
	plugin->priv->not_first_time =
		gconf_client_get_bool (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/not_first_time",
		                       NULL);
	
	plugin->priv->showing_browser =
		gconf_client_get_bool (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/showing_browser",
		                       NULL);
	plugin->priv->browser_size =
		gconf_client_get_int (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/browser_size",
		                       NULL);
	
	plugin->priv->library_browser_visible =
		gconf_client_get_bool (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/library_browser_visible",
		                       NULL);

	plugin->priv->library_browser_pos =
		gconf_client_get_int (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/library_browser_pos",
		                       NULL);
	
	plugin->priv->use_gtk_colors =
		!gconf_client_get_bool (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/use_gtk_colors",
		                       NULL);

	plugin->priv->coverart_size =
		gconf_client_get_int (client,
		                       "/apps/rhythmbox/plugins/rhythmarty/coverart_size",
		                       NULL);
	
	if (plugin->priv->coverart_size==0)
	{
		plugin->priv->coverart_size = 140;	/* Default value */
	}

	const gchar *color;

	color =
		gconf_client_get_string (client,
		                         "/apps/rhythmbox/plugins/rhythmarty/base_color",
		                         NULL);
	if (!color)
	{
		gdk_color_parse("#777777",&plugin->priv->base_color); 	/* Default value */
	}
	else
	{
		gdk_color_parse(color,&plugin->priv->base_color); 
	}

	color =
		gconf_client_get_string (client,
		                         "/apps/rhythmbox/plugins/rhythmarty/text_color",
		                         NULL);
	if (!color)
	{
		gdk_color_parse("#ffffff",&plugin->priv->text_color); 	/* Default value */
	}
	else
	{
		gdk_color_parse(color,&plugin->priv->text_color); 
	}

	color =
		gconf_client_get_string (client,
		                         "/apps/rhythmbox/plugins/rhythmarty/selection_color",
		                         NULL);
	if (!color)
	{
		gdk_color_parse("#658FC4",&plugin->priv->selection_color); 	/* Default value */
	}
	else
	{
		gdk_color_parse(color,&plugin->priv->selection_color); 
	}

	color =
		gconf_client_get_string (client,
		                         "/apps/rhythmbox/plugins/rhythmarty/selection_text_color",
		                         NULL);
	if (!color)
	{
		gdk_color_parse("#ffffff",&plugin->priv->selection_text_color); 	/* Default value */
	}
	else
	{
		gdk_color_parse(color,&plugin->priv->selection_text_color); 
	}
}

static void 
rb_arty_save_configuartion (GConfClient * client,RBArtyPlugin * plugin)
{
	//if not first time
	gconf_client_set_bool(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/not_first_time",
	                      plugin->priv->not_first_time,NULL);
	
	//save browser positions 
	gconf_client_set_bool(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/showing_browser",
	                      plugin->priv->showing_browser,NULL);

	gconf_client_set_int(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/browser_size",
	                      plugin->priv->browser_size,NULL);
	
	gconf_client_set_bool(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/library_browser_visible",
	                      plugin->priv->library_browser_visible,NULL);
	gconf_client_set_int(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/library_browser_pos",
	                      plugin->priv->library_browser_pos,NULL);

	//save configurations
	gconf_client_set_bool(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/use_gtk_colors",
	                      !plugin->priv->use_gtk_colors,NULL);
	gconf_client_set_string(client,
	                        "/apps/rhythmbox/plugins/rhythmarty/base_color",
	                        gdk_color_to_string(&plugin->priv->base_color),
	                        NULL);
	gconf_client_set_string(client,
	                        "/apps/rhythmbox/plugins/rhythmarty/text_color",
	                        gdk_color_to_string(&plugin->priv->text_color),
	                        NULL);
	gconf_client_set_string(client,
	                        "/apps/rhythmbox/plugins/rhythmarty/selection_color",
	                        gdk_color_to_string(&plugin->priv->selection_color),
	                        NULL);
	gconf_client_set_string(client,
	                        "/apps/rhythmbox/plugins/rhythmarty/selection_text_color",
	                        gdk_color_to_string(&plugin->priv->selection_text_color),
	                        NULL);
	gconf_client_set_int(client,
	                      "/apps/rhythmbox/plugins/rhythmarty/coverart_size",
	                      plugin->priv->coverart_size,NULL);

}

//######################### Configuration dialog ###############################
static void
rb_arty_colors_changed(RBArtyPlugin *plugin)
{
	if(plugin->priv->use_gtk_colors)
	{
		//use system theme colors
		GtkStyle *style = gtk_widget_get_style (plugin->priv->genre_view);
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_NORMAL,
		                        &style->base[GTK_STATE_NORMAL]);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_NORMAL,
		                        &style->text[GTK_STATE_NORMAL]);
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_SELECTED,
		                        &style->base[GTK_STATE_SELECTED]);
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_ACTIVE,
		                        &style->base[GTK_STATE_ACTIVE]);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_SELECTED,
		                        &style->text[GTK_STATE_SELECTED]);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_ACTIVE,
		                        &style->text[GTK_STATE_ACTIVE]);
	}
	else
	{
		//use selected colors
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_NORMAL,
		                        &plugin->priv->base_color);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_NORMAL,
		                        &plugin->priv->text_color);
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_SELECTED,
		                        &plugin->priv->selection_color);
		gtk_widget_modify_base (plugin->priv->album_view,
		                        GTK_STATE_ACTIVE,
		                        &plugin->priv->selection_color);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_SELECTED,
		                        &plugin->priv->selection_text_color);
		gtk_widget_modify_text (plugin->priv->album_view,
		                        GTK_STATE_ACTIVE,
		                        &plugin->priv->selection_text_color);
	}
}

static void
rb_arty_use_gtk_color_toggled(GtkToggleButton *togglebutton,
                              RBArtyPlugin *plugin)
{
	plugin->priv->use_gtk_colors = 
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(togglebutton));

	gtk_widget_set_sensitive (plugin->priv->colors_table,!plugin->priv->use_gtk_colors);
	rb_arty_colors_changed(plugin);
}

static void
rb_arty_base_color_changed(GtkColorButton *widget,
                           RBArtyPlugin *plugin)
{
	gtk_color_button_get_color (widget,&plugin->priv->base_color);
	rb_arty_colors_changed(plugin);
}

static void
rb_arty_text_color_changed(GtkColorButton *widget,
                           RBArtyPlugin *plugin)
{
	gtk_color_button_get_color (widget,&plugin->priv->text_color);
	rb_arty_colors_changed(plugin);
}

static void
rb_arty_selection_color_changed(GtkColorButton *widget,
                                RBArtyPlugin *plugin)
{
	gtk_color_button_get_color (widget,&plugin->priv->selection_color);
	rb_arty_colors_changed(plugin);
}

static void
rb_arty_selection_text_color_changed(GtkColorButton *widget,
                                     RBArtyPlugin *plugin)
{
	gtk_color_button_get_color (widget,&plugin->priv->selection_text_color);
	rb_arty_colors_changed(plugin);
}

static void
rb_arty_cover_size_changed(GtkSpinButton *widget,
                           RBArtyPlugin *plugin)
{
	plugin->priv->coverart_size = 
		gtk_spin_button_get_value (GTK_SPIN_BUTTON(widget));

	rb_arty_update_cover_size(plugin);
}

static GtkWidget* 
rb_arty_create_config_widgets(RBArtyPlugin *plugin)
{
	GtkWidget *checkbox = 
		gtk_check_button_new_with_label (_("Use system theme"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbox),
	                                plugin->priv->use_gtk_colors);
	
	GtkWidget *base_label=gtk_label_new (_("Background color: "));
	GtkWidget *text_label=gtk_label_new (_("Text color :"));
	GtkWidget *selection_label=gtk_label_new (_("Selection color: "));
	GtkWidget *selection_text_label=gtk_label_new (_("Selection text color: "));

	GtkWidget *base = gtk_color_button_new_with_color (&plugin->priv->base_color);
	GtkWidget *text = gtk_color_button_new_with_color (&plugin->priv->text_color);
	GtkWidget *selection = gtk_color_button_new_with_color (&plugin->priv->selection_color);
	GtkWidget *selection_text = gtk_color_button_new_with_color (&plugin->priv->selection_text_color);

	GtkWidget *artsize_label = gtk_label_new (_("Cover art size: "));

	GtkAdjustment *adj = (GtkAdjustment*)gtk_adjustment_new (plugin->priv->coverart_size,50,200,
	                                         10,0,0);
	GtkWidget *artsize = gtk_spin_button_new (adj,10,0);

	gtk_misc_set_alignment(GTK_MISC(base_label),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(text_label),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(selection_label),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(selection_text_label),0.0,0.5);
	gtk_misc_set_alignment(GTK_MISC(artsize_label),0.0,0.5);

	GtkWidget *frame = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME(frame),checkbox);
	GtkWidget *table = gtk_table_new (5,2,FALSE);
	gtk_table_attach (GTK_TABLE(table),base_label,0,1,1,2,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),text_label,0,1,2,3,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),selection_label,0,1,3,4,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),selection_text_label,0,1,4,5,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),base,1,2,1,2,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),text,1,2,2,3,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),selection,1,2,3,4,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_table_attach (GTK_TABLE(table),selection_text,1,2,4,5,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,2);
	gtk_container_add (GTK_CONTAINER(frame),table);

	GtkWidget *table2 = gtk_table_new (2,2,FALSE);
	gtk_table_attach (GTK_TABLE(table2),frame,0,2,0,1,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,5);
	gtk_table_attach (GTK_TABLE(table2),artsize_label,0,1,1,2,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,5);
	gtk_table_attach (GTK_TABLE(table2),artsize,1,2,1,2,
	                  GTK_EXPAND | GTK_FILL,GTK_EXPAND | GTK_FILL,10,5);

	gtk_widget_show_all (table2);

	//keep refrence of widgets
	plugin->priv->colors_table = table;
	plugin->priv->use_gtk_colors_button = checkbox;
	plugin->priv->base_color_button = base;
	plugin->priv->text_color_button = text;
	plugin->priv->selection_color_button = selection;
	plugin->priv->selection_text_color_button = selection_text;
	plugin->priv->art_size_entry = artsize;

	gtk_widget_set_sensitive(table,!plugin->priv->use_gtk_colors);
		
	g_signal_connect (G_OBJECT(plugin->priv->use_gtk_colors_button),"toggled",
	                  G_CALLBACK(rb_arty_use_gtk_color_toggled),plugin);
	g_signal_connect (G_OBJECT(plugin->priv->base_color_button),"color-set",
	                  G_CALLBACK(rb_arty_base_color_changed),plugin);
	g_signal_connect (G_OBJECT(plugin->priv->text_color_button),"color-set",
	                  G_CALLBACK(rb_arty_text_color_changed),plugin);
	g_signal_connect (G_OBJECT(plugin->priv->selection_color_button),"color-set",
	                  G_CALLBACK(rb_arty_selection_color_changed),plugin);
	g_signal_connect (G_OBJECT(plugin->priv->selection_text_color_button),"color-set",
	                  G_CALLBACK(rb_arty_selection_text_color_changed),plugin);
	g_signal_connect (G_OBJECT(plugin->priv->art_size_entry),"value-changed",
	                  G_CALLBACK(rb_arty_cover_size_changed),plugin);
	
	return table2;
}

GtkWidget *
rb_arty_create_config_dialog(RBPlugin *pg)
{
	RBArtyPlugin *plugin = RB_ARTY_PLUGIN (pg);
	plugin->priv = RB_ARTY_PLUGIN_GET_PRIVATE (plugin);

	if(!plugin->priv->config_dialog)
	{
		plugin->priv->config_dialog = gtk_dialog_new();
		gtk_window_set_title (GTK_WINDOW(plugin->priv->config_dialog),_("RhythmArty"));
		GtkWidget *close_button = gtk_button_new_with_mnemonic (_("_Close"));
		gtk_widget_show(close_button);
		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(plugin->priv->config_dialog)->action_area),
		                    close_button,FALSE,FALSE,0);

		GtkWidget *config_table = rb_arty_create_config_widgets(plugin);
		gtk_box_pack_start ((GTK_BOX(GTK_DIALOG(plugin->priv->config_dialog)->vbox)),
		                    config_table,FALSE,FALSE,0);
		                    
		g_signal_connect_swapped (G_OBJECT(close_button),
		                  "clicked",G_CALLBACK(gtk_widget_hide),plugin->priv->config_dialog);
		g_signal_connect (G_OBJECT(plugin->priv->config_dialog),
		                  "delete-event",G_CALLBACK(gtk_widget_hide),NULL);
	}

	return plugin->priv->config_dialog;
}

//############################## Plugin initilisation ##########################

RB_PLUGIN_REGISTER(RBArtyPlugin, rb_arty_plugin);

static void
rb_arty_plugin_init (RBArtyPlugin *plugin)
{
	//register stock icons
	rb_arty_stock_icons_init();
}

static void
rb_arty_plugin_finalize (GObject *object)
{
	G_OBJECT_CLASS (rb_arty_plugin_parent_class)->finalize (object);
}

static void
impl_activate (RBPlugin *pg,
	       RBShell *shell)
{	
	RBArtyPlugin *plugin = RB_ARTY_PLUGIN (pg);
	plugin->priv = RB_ARTY_PLUGIN_GET_PRIVATE (plugin);

	plugin->priv->shell = shell;

	g_object_get(shell,"library-source",&(plugin->priv->library_source),NULL);

	//check weather to show genre view , from rhythembox main config 
	GConfClient *client; 
	client = gconf_client_get_default();

	gint val =  gconf_client_get_int 
		(client,"/apps/rhythmbox/ui/library/browser_views",NULL);

	plugin->priv->showing_genre = (val > 0);

	plugin->priv->genre_notify_id =
		gconf_client_notify_add (client, "/apps/rhythmbox/ui/library/browser_views",
		                         (GConfClientNotifyFunc)
		                         rb_arty_genre_key_changed,
		                         plugin, NULL, NULL);
	rb_arty_load_configuartion(client,plugin);
	g_object_unref (G_OBJECT (client));

	//cache directory
	plugin->priv->cache_dir = 
		g_strconcat (g_get_user_cache_dir(),"/",COVER_CACHE_DIRNAME,NULL);
	g_mkdir_with_parents(plugin->priv->cache_dir,0775);

	//load cover cache 
	plugin->priv->cover_fetcher = rb_arty_cover_fetcher_new (plugin);

	//create all widgets
	rb_arty_create_widgets(plugin,shell);
	g_object_ref(plugin->priv->main_widget);

	//create menu/tool show/hide item
	rb_arty_craete_showhide_items(plugin,shell);
	gtk_toggle_action_set_active(plugin->priv->showhide_action,TRUE);

	//get notified when library source is selected
	RBSourceList *source_list;
	g_object_get(shell,"sourcelist",&(source_list),NULL);
	g_signal_connect(G_OBJECT(source_list),"selected",
	                 G_CALLBACK(rb_arty_source_changed),plugin);

	//restore previous states
	gtk_toggle_action_set_active(plugin->priv->showhidebrowser_action,plugin->priv->showing_browser);
	
	//connect button handlers
	g_signal_connect_swapped(G_OBJECT(plugin->priv->fetching_cancel),"clicked",
	                 G_CALLBACK(rb_arty_cover_fetching_cancel),plugin);
	g_signal_connect(G_OBJECT(plugin->priv->show_songs),"clicked",
	                 G_CALLBACK(rb_arty_show_songs_clicked),plugin);
	g_signal_connect(G_OBJECT(plugin->priv->show_albums),"clicked",
	                 G_CALLBACK(rb_arty_show_albums_clicked),plugin);
}

static void
impl_deactivate	(RBPlugin *pg,
		 RBShell *shell)
{
	RBArtyPlugin *plugin = RB_ARTY_PLUGIN (pg);
	plugin->priv = RB_ARTY_PLUGIN_GET_PRIVATE (plugin);

	//hide plugin if it is showing
	if(plugin->priv->showing_plugin)
	{
		rb_arty_showhide_toggled(plugin->priv->showhide_action,plugin);
	}

	//remove gconf notifications
	GConfClient *client; 
	client = gconf_client_get_default();
	gconf_client_notify_remove (client, plugin->priv->genre_notify_id);
	rb_arty_save_configuartion(client,plugin);
	g_object_unref (G_OBJECT (client));

	//destory configuration dialog
	if(plugin->priv->config_dialog)
		gtk_widget_destroy (plugin->priv->config_dialog);

	//stop first time timeout
	if(plugin->priv->first_time_timeout_id)
		g_source_remove (plugin->priv->first_time_timeout_id);
		
	//stop cover fetching
	rb_arty_cover_fetching_cancel(plugin);

	//destroy widgets
	gtk_widget_destroy (plugin->priv->fetching_box);
	gtk_widget_destroy (plugin->priv->show_songs);
	gtk_widget_destroy (plugin->priv->show_albums);
	rb_arty_remove_showhide_items(plugin,shell);
	g_object_unref(plugin->priv->main_widget);

	//save and free cover cache
	rb_arty_cover_fetcher_free (plugin->priv->cover_fetcher);

	g_free(plugin->priv->cache_dir);
}

static void
rb_arty_plugin_class_init (RBArtyPluginClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	RBPluginClass *plugin_class = RB_PLUGIN_CLASS (klass);

	object_class->finalize = rb_arty_plugin_finalize;

	plugin_class->activate = impl_activate;
	plugin_class->deactivate = impl_deactivate;
	plugin_class->create_configure_dialog = rb_arty_create_config_dialog;
	
	g_type_class_add_private (object_class, sizeof (RBArtyPluginPrivate));
}


